// bmsdTools.cpp : implementation file
//
#include "stdafx.h"
#include "GlobalTools.h"
#include "DrawingTools.h"


#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif



/////////////////////////////////////////////////////////////////////////////
// DRAWING TOLLS

DrawingTools::DrawingTools()
{
	COLORREF	PTCol=RGB(79,49,0);			//Pressure Textcolor
	COLORREF	PFCol=RGB(255,216,0);		//Pressure Framecolor
	COLORREF	TTCol=RGB(109,79,0);		//Temperature Textcolor
	COLORREF	TFCol=RGB(255,216,0);		//Temperature Framecolor
	COLORREF	WTCol=RGB_SCHWARZ;			//Wind Textcolor
	COLORREF	WFCol=RGB(255,216,0);		//Wind Framecolor
	COLORREF	WaTCol=RGB_BLAUGRAU;		//Wave Textcolor
	COLORREF	WaFCol=RGB_CYANBLAU;		//Wave Framecolor

	N_date=N_hour=-1;
	initColors=0;
}

DrawingTools::~DrawingTools()
{
}

/*------------------------------------*
 * Draw DataPlane of seleceltetd date *
 *------------------------------------*/

void DrawingTools::DrawSeaData(CDC* pDC, LPRECT rect, time_t reftime, int hour, int details,
								float xscale, float yscale, float xoff, float yoff)
{
char		buf[256];
	int			z,count;
	double		fa;

	if(!initColors) InitTheWorkMatrixes();
	ymapscale=yscale; 	xmapscale=xscale;
	lmapxoff=xoff; lmapyoff=yoff;
	DetailFlags=details;
	
	GetRect(rect);
	
	//create global pen's for symbol drawings
	CPen WWFPen,WWTPen;
	m_logpen.lopnStyle = PS_INSIDEFRAME;
	m_logpen.lopnWidth.x = m_logpen.lopnWidth.y = 1;
	m_logpen.lopnColor = RGB_SCHWARZ;//WTCol;
	if (!WWTPen.CreatePenIndirect(&m_logpen))		return;		
	m_logpen.lopnColor = WFCol;		
	if (!WWFPen.CreatePenIndirect(&m_logpen))		return;
	WFPen=&WWFPen; WTPen=&WWTPen;
	pDC->SetBkMode(TRANSPARENT);


	grid_yy=sizeTotal.cy/myDrawPara.GRIDWIDTH+1;		//Screen-Y / gridwidth  steps
	grid_xx=sizeTotal.cx/myDrawPara.GRIDWIDTH+1;		//Screen-X / gridwidth

	count=grid_yy*grid_xx;
	sc_data.SetSize(count);


if(myBMS_Para.refresch || N_date!=reftime || N_hour!=hour || N_xscale!=xmapscale || N_yscale!=ymapscale || N_yoff!=lmapyoff || N_xoff!=lmapxoff)
	{
	if(N_date!=reftime)	N_SynArray=0;
	N_hour=hour;
	N_date=reftime;
	N_yscale=ymapscale;
	N_xscale=xmapscale;
	N_yoff=lmapyoff;
	N_xoff=lmapxoff;
	
	for(z=0; z<sc_data.GetSize(); z++)
		{
		ObjectOffsetY=sc_data[z].y=((z/grid_xx)*myDrawPara.GRIDWIDTH)+myDrawPara.GRIDMIDDLE;
		ObjectOffsetX=sc_data[z].x=((z%grid_xx)*myDrawPara.GRIDWIDTH)+myDrawPara.GRIDMIDDLE;

		sc_data[z].lon= -(int)(lmapxoff + ((double)(swh - ObjectOffsetX) * xmapscale));
		sc_data[z].lat= -(int)(lmapyoff + ((double)(shh - ObjectOffsetY) * ymapscale));
		}
   
	if(!LoadScreenField(N_date,N_hour,count,sc_data))
		{
		N_date=0;	//prepare for a restart+
		return;
		}
	}

	myBMS_Para.refresch=0;

	if(DetailFlags & (LAY_WAVE|FIG_WTEMP|LAY_WTEMP))
		{
		if(DetailFlags&LAY_WTEMP) DrawPlanes(pDC,PLANE_WTERM);
		}
	
	if(DetailFlags & (LAY_WAVE))
		{
		if(DetailFlags&(LAY_WAVE))	DrawPlaneColLine(pDC,PLANE_HTSGW); //MAP_COL_RIVER
		}
	if(DetailFlags&(LAY_ICE))		DrawPlanes(pDC,PLANE_ICE);
	if(DetailFlags&(FIG_ISO_WTERM)) DrawPlaneLine(pDC,PLANE_WTERM,COL_P_FRAME);
	if(DetailFlags&FIG_WTEMP)
		{
		for(z=0; z<sc_data.GetSize(); z++)
			{
			if(!(sc_data[z].pflag&FRC_VALID)) continue; 
			if(!sc_data[z].wtemp) continue;

			fa=-128.+(double)sc_data[z].wtemp;
			if(myBMS_Para.flag&1) fa=(fa+273.15)*1.8-459.67;
			sprintf(buf,"%d",(int)fa);
			ObjectOffsetY=sc_data[z].y;
			ObjectOffsetX=sc_data[z].x;
			WriteText(pDC,buf,ObjectOffsetX,ObjectOffsetY-myDrawPara.SYMBOLHALFMID,TTCol);
			}
		}

	if((DetailFlags & (FIG_WAVE|LAY_WAVEDIR)) && !(DetailFlags&(LAY_PRESS|LAY_STEMP|LAY_RH)))
		{
		for(z=0; z<sc_data.GetSize(); z++)
			{
			if(!(sc_data[z].pflag&FRC_VALID)) continue; 
			ObjectOffsetY=sc_data[z].y;
			ObjectOffsetX=sc_data[z].x;
			if(!sc_data[z].htsgw && !sc_data[z].wvdir) continue;
			Wave(pDC,z);
			}
		}
}

int DrawingTools::GetRect(LPRECT rect)
{
	CRect	re;
	double	xs,ys;	

	ViewRect=rect;
	sizeTotal=ViewRect.Size();

	swh = sizeTotal.cx*8; shh = sizeTotal.cy*8;
	if((sizeTotal.cx+swh) > (sizeTotal.cy+shh)) ClipMax=sizeTotal.cx+swh;
	else ClipMax=sizeTotal.cy+shh;
	swh = sizeTotal.cx / 2; 	shh = sizeTotal.cy / 2; //set center point
	xs=(double(sizeTotal.cx)/2.)*xmapscale;
	ys=(double(sizeTotal.cy)/2.)*ymapscale;
	Screen_ymin=(short)((double)(-lmapyoff-ys)); //set screen frame
	Screen_xmin=(short)((double)(-lmapxoff-xs));
	Screen_ymax=(short)((double)(-lmapyoff+ys));
	Screen_xmax=(short)((double)(-lmapxoff+xs));
	return NULL;
}

void DrawingTools::DrawSfcData(CDC* pDC, LPRECT rect, time_t reftime, int hour, int details,
								float xscale, float yscale, float xoff, float yoff)
{
	char		buf[256];
	int			z,count;
	double		fa;
	
	if(!initColors) InitTheWorkMatrixes();
	ymapscale=yscale; 	xmapscale=xscale;
	lmapxoff=xoff; lmapyoff=yoff;
	DetailFlags=details;
	

	GetRect(rect);

	//create global pen's for symbol drawings
	CPen WWFPen,WWTPen;
	m_logpen.lopnStyle = PS_INSIDEFRAME;
	m_logpen.lopnWidth.x = m_logpen.lopnWidth.y = 1;
	m_logpen.lopnColor = RGB_SCHWARZ;//WTCol;
	if (!WWTPen.CreatePenIndirect(&m_logpen))		return;		
	m_logpen.lopnColor = WFCol;		
	if (!WWFPen.CreatePenIndirect(&m_logpen))		return;
	WFPen=&WWFPen; WTPen=&WWTPen;
	pDC->SetBkMode(TRANSPARENT);

	grid_yy=sizeTotal.cy/myDrawPara.GRIDWIDTH+1;		//Screen-Y / gridwidth  steps
	grid_xx=sizeTotal.cx/myDrawPara.GRIDWIDTH+1;		//Screen-X / gridwidth

	count=grid_yy*grid_xx;
	sc_data.SetSize(count);


if(myBMS_Para.refresch || N_date!=reftime || N_hour!=hour || N_xscale!=xmapscale || N_yscale!=ymapscale || N_yoff!=lmapyoff || N_xoff!=lmapxoff)
	{
	if(N_date!=reftime)	N_SynArray=0;
	N_hour=hour;
	N_date=reftime;
	N_yscale=ymapscale;
	N_xscale=xmapscale;
	N_yoff=lmapyoff;
	N_xoff=lmapxoff;
	
	for(z=0; z<sc_data.GetSize(); z++)
		{
		ObjectOffsetY=sc_data[z].y=((z/grid_xx)*myDrawPara.GRIDWIDTH)+myDrawPara.GRIDMIDDLE;
		ObjectOffsetX=sc_data[z].x=((z%grid_xx)*myDrawPara.GRIDWIDTH)+myDrawPara.GRIDMIDDLE;

		sc_data[z].lon= -(int)(lmapxoff + ((double)(swh - ObjectOffsetX) * xmapscale));
		sc_data[z].lat= -(int)(lmapyoff + ((double)(shh - ObjectOffsetY) * ymapscale));
		}
   
	if(!LoadScreenField(N_date,N_hour,count,sc_data))
		{
		N_date=0;	//prepare for a restart+
		return;
		}
	}

	myBMS_Para.refresch=0;

	if(DetailFlags & LAY_SYMB_PRECI)
		{
		sprintf(buf,"%s\\CHARTDATA\\WEATHER.LBJ",myBMS_Para.MeteoAccessDir);
		if(OBJFile.Open(buf,CFile::modeRead|CFile::typeBinary, &fex))
			{
			SPointIbuf=(struct LinePoint*)&PointIBuf[0];
			OBJFile.Read(&OBJ_HDR,sizeof(struct BOYHeader));
			}
		}

	if(DetailFlags&(LAY_PRESS)) 	DrawPlanes(pDC,PLANE_PRESS);
	else if(DetailFlags&(LAY_TEMP)) DrawPlanes(pDC,PLANE_TERM);
	else if(DetailFlags&(LAY_STEMP)) DrawPlanes(pDC,PLANE_WTERM);
	else if(DetailFlags&(LAY_RH))	DrawPlanes(pDC,PLANE_RH);

	if(DetailFlags&LAY_CLOUDS) DrawPlanes(pDC,PLANE_CLOUDS);
	else
		{
		if(DetailFlags&LAY_CLOUD_CONV) DrawPlanes(pDC,PLANE_CCLOUDS);
		if(DetailFlags&LAY_CLOUD_BOUND) DrawPlanes(pDC,PLANE_CLOUDS);
		}

	if(DetailFlags&(LAY_PRECI))		DrawPlanes(pDC,PLANE_PRCPT);
	if(DetailFlags&(LAY_SNOW))		DrawPlanes(pDC,PLANE_SNOW);
	if(DetailFlags&(FIG_ISO_BAR)) 	DrawPlaneLine(pDC,PLANE_PRESS,COL_P_FRAME);
	if(DetailFlags&(FIG_ISO_TERM)) 	DrawPlaneLine(pDC,PLANE_TERM,COL_P_FRAME);
	if(DetailFlags&(FIG_ISO_STERM)) DrawPlaneLine(pDC,PLANE_WTERM,COL_P_FRAME);
	if(DetailFlags&(FIG_ISO_RH)) 	DrawPlaneLine(pDC,PLANE_RH,COL_P_FRAME);

	for(z=0; z<sc_data.GetSize(); z++)
		{
		if(!(sc_data[z].pflag&FRC_VALID)) continue; 
		ObjectOffsetX=sc_data[z].x;
		ObjectOffsetY=sc_data[z].y;

		if(DetailFlags & LAY_SYMB_PRECI)// && sc_data[z].pflag)
			{
			SetPreciAndClouds(pDC,z);
			}


		if((DetailFlags & (FIG_WIND|LAY_WIND)) && (sc_data[z].wind || sc_data[z].wdir)) 
			{
			Arrow(pDC,z);
			}

		if(DetailFlags & FIG_TEMP && !(DetailFlags & FIG_WTEMP) && sc_data[z].temp)
			{
			fa=-128.+(double)sc_data[z].temp;
			if(myBMS_Para.flag&1) fa=(fa+273.15)*1.8-459.67;
			sprintf(buf,"%d",(int)fa);
			WriteText(pDC,buf,ObjectOffsetX,ObjectOffsetY-myDrawPara.SYMBOLHALFMID,TTCol);
			}

		if(DetailFlags & FIG_STEMP && sc_data[z].wtemp)
			{
			fa=-128.+(double)sc_data[z].wtemp;
			if(myBMS_Para.flag&1) fa=(fa+273.15)*1.8-459.67;
			sprintf(buf,"%d",(int)fa);
			WriteText(pDC,buf,ObjectOffsetX,ObjectOffsetY-myDrawPara.SYMBOLHALFMID,TTCol);
			}


		if(DetailFlags & FIG_PRESS && sc_data[z].prmsl)
			{
			sprintf(buf,"%d",sc_data[z].prmsl+900); 
			WriteText(pDC,buf,ObjectOffsetX,ObjectOffsetY+myDrawPara.SYMBOLHALFMID,PTCol);
			}
		if(DetailFlags & FIG_RH && sc_data[z].rh)
			{
			sprintf(buf,"%d",sc_data[z].rh); 
			WriteText(pDC,buf,ObjectOffsetX,ObjectOffsetY,TTCol);
			}
		if(DetailFlags & FIG_PRECI && sc_data[z].prcpt)
			{
			sprintf(buf,"%.1f",(float)sc_data[z].prcpt*.1f); 
			WriteText(pDC,buf,ObjectOffsetX,ObjectOffsetY,TTCol);
			}
		if(DetailFlags & FIG_SNOW && sc_data[z].weasd)
			{
			sprintf(buf,"%.1f",(float)sc_data[z].weasd); 
			WriteText(pDC,buf,ObjectOffsetX,ObjectOffsetY,TTCol);
			}
		}

	if(DetailFlags & LAY_SYMB_PRECI) OBJFile.Close();
}

void DrawingTools::DrawSynData(CDC* pDC, LPRECT rect, time_t reftime, int hour, int details,
								float xscale, float yscale, float xoff, float yoff)
{

	char		buf[256];
	int			z,count;
	
	SynopDetailFlags=details;

	if(hour!=-1) return;
	if(!(SynopDetailFlags & SYNOP_ONOFF)) return;
	if(!(SynopDetailFlags&(SYNOP_WIND|SYNOP_TEMP|SYNOP_PRESS|SYNOP_CLOUDS))) return;





	if(!initColors) InitTheWorkMatrixes();
	ymapscale=yscale; 	xmapscale=xscale;
	lmapxoff=xoff; lmapyoff=yoff;

	

	GetRect(rect);

	//create global pen's for symbol drawings
	CPen WWFPen,WWTPen;
	m_logpen.lopnStyle = PS_INSIDEFRAME;
	m_logpen.lopnWidth.x = m_logpen.lopnWidth.y = 1;
	m_logpen.lopnColor = RGB_SCHWARZ;//WTCol;
	if (!WWTPen.CreatePenIndirect(&m_logpen))		return;		
	m_logpen.lopnColor = RGB_WEISS;		
	if (!WWFPen.CreatePenIndirect(&m_logpen))		return;
	WFPen=&WWFPen; WTPen=&WWTPen;
	pDC->SetBkMode(TRANSPARENT);

	grid_yy=sizeTotal.cy/myDrawPara.GRIDWIDTH+1;		//Screen-Y / gridwidth  steps
	grid_xx=sizeTotal.cx/myDrawPara.GRIDWIDTH+1;		//Screen-X / gridwidth

	count=grid_yy*grid_xx;
	sc_data.SetSize(count);

if(myBMS_Para.refresch || N_date!=reftime || N_hour!=hour || N_xscale!=xmapscale || N_yscale!=ymapscale || N_yoff!=lmapyoff || N_xoff!=lmapxoff)
	{
	if(N_date!=reftime)	N_SynArray=0;

	N_hour=hour;
	N_date=reftime;
	N_yscale=ymapscale;
	N_xscale=xmapscale;
	N_yoff=lmapyoff;
	N_xoff=lmapxoff;
	
	for(z=0; z<sc_data.GetSize(); z++)
		{
		ObjectOffsetY=sc_data[z].y=((z/grid_xx)*myDrawPara.GRIDWIDTH)+myDrawPara.GRIDMIDDLE;
		ObjectOffsetX=sc_data[z].x=((z%grid_xx)*myDrawPara.GRIDWIDTH)+myDrawPara.GRIDMIDDLE;
		sc_data[z].lon= -(int)(lmapxoff + ((double)(swh - ObjectOffsetX) * xmapscale));
		sc_data[z].lat= -(int)(lmapyoff + ((double)(shh - ObjectOffsetY) * ymapscale));
		}
   	
	if(!LoadScreenField(N_date,N_hour-6,count,sc_data))
		{
		N_date=0;	//prepare for a restart+
		return;
		}
	}

	myBMS_Para.refresch=0;

//if(!N_SynArray)
	{
	N_SynArray=SetSynopField(reftime);
	if(!N_SynArray) return;
	}

	sprintf(buf,"%s\\CHARTDATA\\WEATHER.LBJ",myBMS_Para.MeteoAccessDir);
	if(OBJFile.Open(buf,CFile::modeRead|CFile::typeBinary, &fex))
		{
		SPointIbuf=(struct LinePoint*)&PointIBuf[0];
		OBJFile.Read(&OBJ_HDR,sizeof(struct BOYHeader));
		}

	if(SynopDetailFlags&SYNOP_GRID)	
		{
		for(z=0; z<sc_data.GetSize(); z++)
			{
			if(!(sc_data[z].pflag&FRC_VALID)) continue; 
			ObjectOffsetX=sc_data[z].x;
			ObjectOffsetY=sc_data[z].y;
			SetDetailSympol(pDC,sc_data[z].pflag,sc_data[z].prmsl,sc_data[z].btcdc,sc_data[z].temp,z);
			}
		}

	else
	for(z=0; z<mySynopList.GetSize(); z++)
		{
		ObjectOffsetX = (int)(  (double) (((double)mySynopList[z].lon + lmapxoff) / xmapscale)) + swh;
		ObjectOffsetY = (int)(  (double) (((double)mySynopList[z].lat + lmapyoff) / ymapscale)) + shh;
		SetDetailSympol(pDC,mySynopList[z].data.pflag,
			mySynopList[z].data.prmsl,
			mySynopList[z].data.btcdc,
			mySynopList[z].data.temp,-1,
			mySynopList[z].data.ugrd,
			mySynopList[z].data.vgrd);
		}


	if(SynopDetailFlags & SYNOP_ONOFF) OBJFile.Close();


}

void DrawingTools::SetDetailSympol(CDC* pDC,int rflags, int prmsl, int btcdc, int temp,int z, int ugrd, int vgrd)
{
	char	buf[32];
	double		fa;

	if(SynopDetailFlags & SYNOP_CLOUDS)  Precipitation(pDC,rflags,btcdc);
	if(SynopDetailFlags&SYNOP_WIND)
		{
		if(z!=-1) SynGridArrow(pDC,z);
		else if(ugrd && vgrd) SynGridArrow(pDC,ugrd,vgrd);
		}

	if(SynopDetailFlags & SYNOP_TEMP  && temp)
		{
		if(z==-1) fa=( (double)temp /10.+.5);
		else fa=-128.+(double)sc_data[z].temp;
		if(myBMS_Para.flag&1) fa=(fa+273.15)*1.8-459.67;
		sprintf(buf,"%d",(int)fa);
		WriteText(pDC,buf,ObjectOffsetX,ObjectOffsetY-myDrawPara.SYMBOLTOP-4,TTCol);
		}

	if(SynopDetailFlags & SYNOP_PRESS && prmsl)
		{
		if(z==-1) 
			{
			prmsl=(int)((double)prmsl/10.+.5);
			sprintf(buf,"%d",prmsl+900); 
			}
		else sprintf(buf,"%d",sc_data[z].prmsl+900); 
		WriteText(pDC,buf,ObjectOffsetX,ObjectOffsetY+myDrawPara.SYMBOLTOP-1,PTCol);
		}
}

void DrawingTools::Precipitation(CDC* pDC, int rflags, int btcdc)
{
	if(rflags&FRC_RAIN) {PlotOjects(pDC,OBJ_RAIN); return;}
	if(rflags&FRC_ICEP) {PlotOjects(pDC,OBJ_HAIL); return;}
	if(rflags&FRC_SNOW) {PlotOjects(pDC,OBJ_SNOW); return;}

	if(btcdc!=128)
		{
		if(btcdc<20) {PlotOjects(pDC,OBJ_SUN); return;}
		if(btcdc>70) {PlotOjects(pDC,OBJ_CLOUDS); return;}
		PlotOjects(pDC,OBJ_CLOUDSUN);
		}
}

#include <math.h>
/*---------------------------------------------*
 * Drawing Arrows for Synop Grid WindDirection *
 *---------------------------------------------*/
void DrawingTools::SynGridArrow(CDC* pDC,int z)
{
	char		buf[32];
	COLORREF	lcol=WTCol;
	if(!(DetailFlags&LAY_COL_WIND))
		{
		if(DetailFlags&LAY_PRESS) lcol=myDrawMatrix[sc_data[z].prmsl+32].col.press;
		else if(DetailFlags&LAY_TEMP) lcol=myDrawMatrix[sc_data[z].temp+32].col.airtemp;
		else if(DetailFlags&(LAY_WTEMP|LAY_STEMP)) lcol=myDrawMatrix[sc_data[z].wtemp+32].col.sfctemp;
		}



//	if(DetailFlags&LAY_WIND)
		{	
		CPen Pen;
		m_logpen.lopnStyle = PS_INSIDEFRAME;
		m_logpen.lopnWidth.x = m_logpen.lopnWidth.y = 1;
		m_logpen.lopnColor = lcol; //RGB(20,240,240); //myDrawMatrix[sc_data[z].wind].windK;
		if (!Pen.CreatePenIndirect(&m_logpen))		return;	

		CPen* pOldPen=pDC->SelectObject(&Pen);

		pDC->MoveTo(myDrawMatrix[(int)sc_data[z].wdir].windx1+ObjectOffsetX,myDrawMatrix[(int)sc_data[z].wdir].windy1+ObjectOffsetY); 
		pDC->LineTo(myDrawMatrix[(int)sc_data[z].wdir].windx2+ObjectOffsetX,myDrawMatrix[(int)sc_data[z].wdir].windy2+ObjectOffsetY);

		pDC->MoveTo(myDrawMatrix[(int)sc_data[z].wdir].windx2+ObjectOffsetX,myDrawMatrix[(int)sc_data[z].wdir].windy2+ObjectOffsetY); 
		pDC->LineTo(myDrawMatrix[(int)sc_data[z].wdir].windx3+ObjectOffsetX,myDrawMatrix[(int)sc_data[z].wdir].windy3+ObjectOffsetY);

		pDC->MoveTo(myDrawMatrix[(int)sc_data[z].wdir].windx2+ObjectOffsetX,myDrawMatrix[(int)sc_data[z].wdir].windy2+ObjectOffsetY); 
		pDC->LineTo(myDrawMatrix[(int)sc_data[z].wdir].windx4+ObjectOffsetX,myDrawMatrix[(int)sc_data[z].wdir].windy4+ObjectOffsetY);
		pDC->SelectObject(&pOldPen);
		}

	if(!sc_data[z].wind) return;

	sprintf(buf,"%2d",myDrawMatrix[(int)sc_data[z].wind].beaufort);
	DoppelText(pDC,buf,ObjectOffsetX,ObjectOffsetY,WTCol,RGB_WEISS);

}


void DrawingTools::SynGridArrow(CDC* pDC, int ugrd, int vgrd)
{
	char		buf[32];
	_complex	uv;
	COLORREF	lcol=WTCol;

	uv.x=((double)ugrd/100.); 
	uv.y=((double)vgrd/100.);

	double		wi,wd;

	wi=(USHORT)(_cabs(uv)*10.);
	wd=(USHORT)((180+(atan2(uv.x,uv.y)/RAD))*10.);
	BYTE wind= (BYTE)((double)wi/2.5+.5);
	BYTE wdir= (BYTE)(((double)wd/14.0625+.5));

	if(!(DetailFlags&LAY_COL_WIND))
		{
		if(DetailFlags&LAY_PRESS) lcol=myDrawMatrix[sc_data[wind].prmsl+32].col.press;
		else if(DetailFlags&LAY_TEMP) lcol=myDrawMatrix[sc_data[wind].temp+32].col.airtemp;
		else if(DetailFlags&(LAY_WTEMP|LAY_STEMP)) lcol=myDrawMatrix[sc_data[wind].wtemp+32].col.sfctemp;
		}

	CPen Pen;
	m_logpen.lopnStyle = PS_INSIDEFRAME;
	m_logpen.lopnWidth.x = m_logpen.lopnWidth.y = 1;
	m_logpen.lopnColor = WTCol; 
	if (!Pen.CreatePenIndirect(&m_logpen))		return;	

	CPen* pOldPen=pDC->SelectObject(&Pen);

	pDC->MoveTo(myDrawMatrix[wdir].windx1+ ObjectOffsetX,myDrawMatrix[wdir].windy1+ObjectOffsetY);
	pDC->LineTo(myDrawMatrix[wdir].windx2+ObjectOffsetX,myDrawMatrix[wdir].windy2+ObjectOffsetY);

	pDC->MoveTo(myDrawMatrix[wdir].windx2+ObjectOffsetX,myDrawMatrix[wdir].windy2+ObjectOffsetY); 
	pDC->LineTo(myDrawMatrix[wdir].windx3+ObjectOffsetX,myDrawMatrix[wdir].windy3+ObjectOffsetY);

	pDC->MoveTo(myDrawMatrix[wdir].windx2+ObjectOffsetX,myDrawMatrix[wdir].windy2+ObjectOffsetY); 
	pDC->LineTo(myDrawMatrix[wdir].windx4+ObjectOffsetX,myDrawMatrix[wdir].windy4+ObjectOffsetY);
	pDC->SelectObject(&pOldPen);

	if(!wind) return;

	sprintf(buf,"%2d",myDrawMatrix[wind].beaufort);
	DoppelText(pDC,buf,ObjectOffsetX,ObjectOffsetY,WTCol,RGB_WEISS);
}

void DrawingTools::SetPreciAndClouds(CDC* pDC,int z)
{
	if(sc_data[z].pflag&(FRC_RAIN)) 
		{
		PlotOjects(pDC,OBJ_RAIN); return;
		}
	if(sc_data[z].pflag&FRC_ICEP) {PlotOjects(pDC,OBJ_HAIL); return;}
	if(sc_data[z].pflag&FRC_SNOW) {PlotOjects(pDC,OBJ_SNOW); return;}

//	if(sc_data[z].btcdc>70) {PlotOjects(pDC,OBJ_CLOUDS); return;}
//	if(sc_data[z].btcdc>30) {PlotOjects(pDC,OBJ_CLOUDSUN); return;}
}


/*-------------------------*
 * draw wind and direction *
 *-------------------------*/
void DrawingTools::Arrow(CDC* pDC,int z)
{
	char		buf[32];
	COLORREF	lcol=WTCol;
	if(!(DetailFlags&LAY_COL_WIND))
		{
		if(DetailFlags&LAY_PRESS) lcol=myDrawMatrix[sc_data[z].prmsl+32].col.press;
		else if(DetailFlags&LAY_TEMP) lcol=myDrawMatrix[sc_data[z].temp+32].col.airtemp;
		else if(DetailFlags&(LAY_WTEMP|LAY_STEMP)) lcol=myDrawMatrix[sc_data[z].wtemp+32].col.sfctemp;
		}
	else 
		{
		lcol=myDrawMatrix[myDrawMatrix[sc_data[z].wind].beaufort].col.wind;
		}


	if(DetailFlags&LAY_WIND)
		{	
		CPen Pen;
		m_logpen.lopnStyle = PS_INSIDEFRAME;
		m_logpen.lopnWidth.x = m_logpen.lopnWidth.y = 1;
		m_logpen.lopnColor = lcol; //RGB(20,240,240); //myDrawMatrix[sc_data[z].wind].windK;
		if (!Pen.CreatePenIndirect(&m_logpen))		return;	

		CPen* pOldPen=pDC->SelectObject(&Pen);

		pDC->MoveTo(myDrawMatrix[(int)sc_data[z].wdir].windx1+ObjectOffsetX,myDrawMatrix[(int)sc_data[z].wdir].windy1+ObjectOffsetY); 
		if(DetailFlags&FIG_WIND && sc_data[z].wind)
			{
			pDC->LineTo(myDrawMatrix[(int)sc_data[z].wdir].windx1a+ObjectOffsetX,myDrawMatrix[(int)sc_data[z].wdir].windy1a+ObjectOffsetY);
			pDC->MoveTo(myDrawMatrix[(int)sc_data[z].wdir].windx1b+ObjectOffsetX,myDrawMatrix[(int)sc_data[z].wdir].windy1b+ObjectOffsetY); 
			}
		pDC->LineTo(myDrawMatrix[(int)sc_data[z].wdir].windx2+ObjectOffsetX,myDrawMatrix[(int)sc_data[z].wdir].windy2+ObjectOffsetY);

		pDC->MoveTo(myDrawMatrix[(int)sc_data[z].wdir].windx2+ObjectOffsetX,myDrawMatrix[(int)sc_data[z].wdir].windy2+ObjectOffsetY); 
		pDC->LineTo(myDrawMatrix[(int)sc_data[z].wdir].windx3+ObjectOffsetX,myDrawMatrix[(int)sc_data[z].wdir].windy3+ObjectOffsetY);

		pDC->MoveTo(myDrawMatrix[(int)sc_data[z].wdir].windx2+ObjectOffsetX,myDrawMatrix[(int)sc_data[z].wdir].windy2+ObjectOffsetY); 
		pDC->LineTo(myDrawMatrix[(int)sc_data[z].wdir].windx4+ObjectOffsetX,myDrawMatrix[(int)sc_data[z].wdir].windy4+ObjectOffsetY);
		pDC->SelectObject(&pOldPen);
		}

	if(!sc_data[z].wind) return;

	if(DetailFlags&FIG_WIND)
		{
		sprintf(buf,"%2d",myDrawMatrix[(int)sc_data[z].wind].beaufort);
		WriteText(pDC,buf,ObjectOffsetX,ObjectOffsetY,lcol);
		}

}

/*--------------------------------*
 * draw wave height and direction *
 *--------------------------------*/
void DrawingTools::Wave(CDC* pDC,int z)
{
	char		buf[32];
	COLORREF	lcol=WaFCol;

	if(DetailFlags&LAY_PRESS) lcol=myDrawMatrix[sc_data[z].prmsl+32].col.press;
	else if(DetailFlags&LAY_TEMP) lcol=myDrawMatrix[sc_data[z].temp+32].col.airtemp;
	else if(DetailFlags&FIG_WAVE) 
		{
		lcol=WaTCol;
		}

	lcol=myDrawMatrix[sc_data[z].htsgw].col.waves;

	if(DetailFlags&LAY_WAVEDIR )
		{	

		CPen Pen;
		m_logpen.lopnStyle = PS_INSIDEFRAME;
		m_logpen.lopnWidth.x = m_logpen.lopnWidth.y = 1;
		m_logpen.lopnColor = lcol; 
		if (!Pen.CreatePenIndirect(&m_logpen))		return;	

		CPen* pOldPen=pDC->SelectObject(&Pen);

		pDC->MoveTo(myDrawMatrix[(int)sc_data[z].wvdir].wavex1+ObjectOffsetX,myDrawMatrix[(int)sc_data[z].wvdir].wavey1+ObjectOffsetY); 
		if(DetailFlags&FIG_WAVE)
			{
			pDC->LineTo(myDrawMatrix[(int)sc_data[z].wvdir].wavex1a+ObjectOffsetX,myDrawMatrix[(int)sc_data[z].wvdir].wavey1a+ObjectOffsetY); 
			pDC->MoveTo(myDrawMatrix[(int)sc_data[z].wvdir].wavex1b+ObjectOffsetX,myDrawMatrix[(int)sc_data[z].wvdir].wavey1b+ObjectOffsetY);
			}
		pDC->LineTo(myDrawMatrix[(int)sc_data[z].wvdir].wavex2+ObjectOffsetX,myDrawMatrix[(int)sc_data[z].wvdir].wavey2+ObjectOffsetY);

		pDC->MoveTo(myDrawMatrix[(int)sc_data[z].wvdir].wavex2+ObjectOffsetX,myDrawMatrix[(int)sc_data[z].wvdir].wavey2+ObjectOffsetY); 
		pDC->LineTo(myDrawMatrix[(int)sc_data[z].wvdir].wavex3+ObjectOffsetX,myDrawMatrix[(int)sc_data[z].wvdir].wavey3+ObjectOffsetY);

		pDC->MoveTo(myDrawMatrix[(int)sc_data[z].wvdir].wavex2+ObjectOffsetX,myDrawMatrix[(int)sc_data[z].wvdir].wavey2+ObjectOffsetY); 
		pDC->LineTo(myDrawMatrix[(int)sc_data[z].wvdir].wavex4+ObjectOffsetX,myDrawMatrix[(int)sc_data[z].wvdir].wavey4+ObjectOffsetY);
		pDC->SelectObject(&pOldPen);
		}


	if(!sc_data[z].htsgw) return;

	if(DetailFlags&FIG_WAVE)
		{
		sprintf(buf,"%d",(int)((double)sc_data[z].htsgw*.25+.5));
		WriteText(pDC,buf,ObjectOffsetX,ObjectOffsetY,lcol);
		}

}


void DrawingTools::WriteText(CDC* pDC,char *txt, int x, int y, COLORREF TCol)
{
	RECT	Rect;
	long	len,xs,ys,xe,ye;

	len=(long)strlen(txt)*8;
	xs=(long)x-len; xe=(long)x+len; ys=(long)(y-8); ye=(long)(y+8);

	pDC->SetTextColor(TCol);

	Rect.left=xs; Rect.right=xe; Rect.top=ys; Rect.bottom=ye;
	pDC->DrawText(txt,&Rect,DT_CENTER | DT_VCENTER | DT_SINGLELINE);
}

void DrawingTools::DoppelText(CDC* pDC,char *txt, int x, int y, COLORREF TCol, COLORREF FCol)
{
	RECT	Rect;
	long	i,j,len,xs,ys,xe,ye;

	len=(long)strlen(txt)*12;
	xs=(long)x-len; xe=(long)x+len; ys=(long)(y-8); ye=(long)(y+8);

	pDC->SetTextColor(FCol);

	for(i=-1; i<=1; i++)
		{
        for(j=-1; j<=1; j++)
			{
            if(!(i | j)) continue;
			Rect.left=xs+i; Rect.right=xe+i; Rect.top=ys+j; Rect.bottom=ye+j;
			pDC->DrawText(txt,&Rect,DT_CENTER | DT_VCENTER | DT_SINGLELINE);
		    }
		}
	pDC->SetTextColor(TCol);

	Rect.left=xs; Rect.right=xe; Rect.top=ys; Rect.bottom=ye;
	pDC->DrawText(txt,&Rect,DT_CENTER | DT_VCENTER | DT_SINGLELINE);
}
void DrawingTools::DrawPlanes(CDC* pDC, int type)
{
	int			i,z,end,start;
	COLORREF	bcol,pcol;

	if(!myPoliLines.GetSize()) return;
	if(!myCoordinates.GetSize()) return;
	if(type>(myIsoPlanes.GetSize()-1)) return;

	if(type==PLANE_CLOUDS && DetailFlags&LAY_CLOUDS && !(DetailFlags&LAY_PRECI) )
		{
		end=myIsoPlanes[PLANE_PRCPT].start+myIsoPlanes[PLANE_PRCPT].len;
		if(end>myPoliLines.GetSize()) end=myPoliLines.GetSize();
		start=myIsoPlanes[PLANE_PRCPT].start;

		//precipitation have allways clouds
		for(i=start; i<end; i++)
			{
			if(i<0 || i>=myPoliLines.GetSize()) break;
			if(myPoliLines[i].type!=PLANE_PRCPT) continue;
			if(myPoliLines[i].value>26) continue;
			if( myPoliLines[i].ymax < Screen_ymin) continue;
			if((myPoliLines[i].xmax < Screen_xmin)
			||(myPoliLines[i].xmin > Screen_xmax)) continue;
			if( myPoliLines[i].ymin > Screen_ymax) continue;
			pcol=bcol=myDrawMatrix[(myPoliLines[i].value/2)*10].col.clouds;
			DrawIsoPolygon(pDC,&myCoordinates[myPoliLines[i].start],myPoliLines[i].len,bcol,pcol,1);
			}
		}


	end=myIsoPlanes[type].start+myIsoPlanes[type].len;
	if(end>myPoliLines.GetSize()) end=myPoliLines.GetSize();
	start=myIsoPlanes[type].start;

	for(i=start; i<end; i++)
		{
		if(i<0 || i>=myPoliLines.GetSize()) break;
		if(myPoliLines[i].type!=type) continue;

		if( myPoliLines[i].ymax < Screen_ymin) continue;
		if((myPoliLines[i].xmax < Screen_xmin)
		||(myPoliLines[i].xmin > Screen_xmax)) continue;
		if( myPoliLines[i].ymin > Screen_ymax) continue;

		z=myPoliLines[i].value;
		z&=255;

		if((type==PLANE_CLOUDS || type==PLANE_CCLOUDS || type==PLANE_PRCPT) && !z) continue;

		switch(type)
			{
			case	PLANE_PRESS:	bcol=myDrawMatrix[z].col.press;
									z+=myBMS_Para.iso_width[type]*2;
									z&=255;
									pcol=myDrawMatrix[z].col.press;
									break;

			case	PLANE_TERM:		pcol=bcol=myDrawMatrix[z].col.airtemp;
									break;

			case	PLANE_WTERM:	pcol=bcol=myDrawMatrix[z].col.sfctemp;
									break;

			case	PLANE_HTSGW:	pcol=bcol=myDrawMatrix[z].col.waves;
									break;

			case	PLANE_CCLOUDS:	pcol=bcol=myDrawMatrix[z+8].col.clouds;
									break;
			case	PLANE_CLOUDS:	pcol=bcol=myDrawMatrix[z].col.clouds;
									break;

			case	PLANE_PRCPT:	pcol=bcol=myDrawMatrix[z].col.prcpt;
									break;
			
			case	PLANE_SNOW:		pcol=bcol=myDrawMatrix[z].col.weasd;
									break;
			
			case	PLANE_ICE:		pcol=bcol=RGB_WEISS; //withe
									break;

			case	PLANE_RH:		pcol=bcol=myDrawMatrix[z].col.rh;
									break;

			}
		
		DrawIsoPolygon(pDC,&myCoordinates[myPoliLines[i].start],myPoliLines[i].len,bcol,pcol,1);
		}
}

void DrawingTools::DrawPlaneLine(CDC* pDC, int type, COLORREF col)
{
	int			i,end,start;

	if(!myPoliLines.GetSize()) return;
	if(!myCoordinates.GetSize()) return;
	if(type>(myIsoPlanes.GetSize()-1)) return;

	end=myIsoPlanes[type].start+myIsoPlanes[type].len;
	if(end>myPoliLines.GetSize()) end=myPoliLines.GetSize();
	start=myIsoPlanes[type].start;
	for(i=start; i<end; i++)
		{
		if(myPoliLines[i].type==type)
			{
			if( myPoliLines[i].ymax < Screen_ymin) continue;
			if((myPoliLines[i].xmax < Screen_xmin)
			||(myPoliLines[i].xmin > Screen_xmax)) continue;
			if( myPoliLines[i].ymin > Screen_ymax) continue;
			DrawIsoLine(pDC,&myCoordinates[myPoliLines[i].start],myPoliLines[i].len,col);
			}
		}
}



void DrawingTools::DrawPlaneColLine(CDC* pDC, int type)
{
	int			z,i,end,start;
	COLORREF	col;

	if(!myPoliLines.GetSize()) return;
	if(!myCoordinates.GetSize()) return;

	end=myIsoPlanes[type].start+myIsoPlanes[type].len;
	if(end>myPoliLines.GetSize()) end=myPoliLines.GetSize();
	start=myIsoPlanes[type].start;

	for(i=start; i<end; i++)
		{
		if( myPoliLines[i].ymax < Screen_ymin) continue;
		if((myPoliLines[i].xmax < Screen_xmin)
		||(myPoliLines[i].xmin > Screen_xmax)) continue;
		if( myPoliLines[i].ymin > Screen_ymax) continue;

		z=myPoliLines[i].value;
		switch(myPoliLines[i].type)
			{
			case	PLANE_PRESS:	col=myDrawMatrix[z].col.press;						
									break;
			case	PLANE_TERM:		col=myDrawMatrix[z].col.airtemp;
									break;
			case	PLANE_WTERM:	col=myDrawMatrix[z].col.sfctemp;
									break;
			case	PLANE_HTSGW:	col=myDrawMatrix[z].col.waves;
									break;
			case	PLANE_CCLOUDS:
			case	PLANE_CLOUDS:	col=myDrawMatrix[z].col.clouds;
									break;
			case	PLANE_PRCPT:	col=myDrawMatrix[z].col.prcpt;
									break;
			case	PLANE_SNOW:		col=myDrawMatrix[z].col.weasd;
									break;
			case	PLANE_ICE:		col=RGB_WEISS; //withe
									break;
			case	PLANE_RH:		col=myDrawMatrix[z].col.rh;
									break;

			}
		DrawIsoLine(pDC,&myCoordinates[myPoliLines[i].start],myPoliLines[i].len,col);
		}
}


void DrawingTools::DrawIsoLine(CDC* pDC, COORDIANTE *point, int count, COLORREF col)
{
	if(count<1) return;

	int		i;
	CPen Pen(PS_SOLID, 1, col);
	CPen*pOldPen=pDC->SelectObject(&Pen);

	if(count<1) return;
	PolyPoints mPoints;
	mPoints.SetSize(count);
	for (i=0; i<count; i++) //CalcPoint(&point[i], &mPoints[i]);
		{
		mPoints[i].x=(int)((double)(((double)point[i].x + lmapxoff)/xmapscale))+swh;
		mPoints[i].y=(int)((double)(((double)point[i].y + lmapyoff)/ymapscale))+shh;
		if(mPoints[i].x>ClipMax)mPoints[i].x=ClipMax;
		if(mPoints[i].x<-ClipMax)mPoints[i].x=-ClipMax;
		if(mPoints[i].y>ClipMax)mPoints[i].y=ClipMax;
		if(mPoints[i].y<-ClipMax)mPoints[i].y=-ClipMax;
		}

	pDC->Polyline(mPoints.GetData(),count);
	pDC->SelectObject(pOldPen);
}

//if you want then calulate self
void DrawingTools::CalcPoint(COORDIANTE *in_coor, POINT *out_point)
{
	out_point->x=(int)((double)(((double)in_coor->x + lmapxoff)/xmapscale))+swh;
	out_point->y=(int)((double)(((double)in_coor->y + lmapyoff)/ymapscale))+shh;
	if(out_point->x>ClipMax)out_point->x=ClipMax;
	if(out_point->x<-ClipMax)out_point->x=-ClipMax;
	if(out_point->y>ClipMax)out_point->y=ClipMax;
	if(out_point->y<-ClipMax)out_point->y=-ClipMax;
}


void DrawingTools::DrawIsoPolygon(CDC* pDC, COORDIANTE *point, int count, COLORREF bcol, COLORREF pcol, int flag)
{
	CBrush	Brush(bcol);
	CBrush	*pOldBrush=pDC->SelectObject(&Brush);
	CPen	Pen(PS_SOLID, 1,pcol);
	CPen	*pOldPen;
	if(flag && bcol!=pcol) pOldPen=pDC->SelectObject(&Pen);
	else pOldPen=(CPen*) pDC->SelectStockObject(NULL_PEN);


	pDC->SetPolyFillMode(ALTERNATE);


	PolyPoints mPoints;
	mPoints.SetSize(count);
	for (int i=0; i<count; i++) //CalcPoint(&point[i], &mPoints[i]);
		{
		mPoints[i].x=(int)((double)(((double)point[i].x + lmapxoff)/xmapscale))+swh;
		mPoints[i].y=(int)((double)(((double)point[i].y + lmapyoff)/ymapscale))+shh;
		if(mPoints[i].x>ClipMax)mPoints[i].x=ClipMax;
		if(mPoints[i].x<-ClipMax)mPoints[i].x=-ClipMax;
		if(mPoints[i].y>ClipMax)mPoints[i].y=ClipMax;
		if(mPoints[i].y<-ClipMax)mPoints[i].y=-ClipMax;
		}

	pDC->Polygon(mPoints.GetData(),count);

	pDC->SelectObject(pOldBrush);
	pDC->SelectObject(pOldPen);
}

void DrawingTools::InitTheWorkMatrixes()
{
	double		w,dir;

	if(!initColors)
		{
		initColors=1; 
		SetGridSize(myBMS_Para.GridSize,0); //do not force this InitTheWorkMatrixes
		}

	myDrawMatrix.SetSize(256);	// set the size of the matrix to 256  
	ColorMatrix& myColors=GetColorMatrix();

	for(int i=0; i<256; i++)
		{
		myDrawMatrix[i].col.press	=myColors[i].press;	
		myDrawMatrix[i].col.airtemp	=myColors[i].airtemp;
		myDrawMatrix[i].col.wind	=myColors[i].wind;
		myDrawMatrix[i].col.waves	=myColors[i].waves;
		myDrawMatrix[i].col.clouds	=myColors[i].clouds;
		myDrawMatrix[i].col.prcpt	=myColors[i].prcpt;
		myDrawMatrix[i].col.weasd	=myColors[i].weasd;
		myDrawMatrix[i].col.rh		=myColors[i].rh;
		myDrawMatrix[i].col.sfctemp	=myColors[i].sfctemp;
		myDrawMatrix[i].col.bathy	=myColors[i].bathy;


		dir= (180.-(double)i*1.40625);	
		w  = dir * RAD; 
		myDrawMatrix[i].wavex1 = myDrawMatrix[i].windx1 = (int)((sin(w) * myDrawPara.ARROW_LEN)+.5); 
		myDrawMatrix[i].wavey1 = myDrawMatrix[i].windy1 = (int)((cos(w) * myDrawPara.ARROW_LEN)+.5);
		myDrawMatrix[i].wavex1a = myDrawMatrix[i].windx1a = (int)((sin(w) * myDrawPara.ARROW_LEN_MID)+.5);
		myDrawMatrix[i].wavey1a = myDrawMatrix[i].windy1a = (int)((cos(w) * myDrawPara.ARROW_LEN_MID)+.5);

		w  = (double)(dir-180.) * RAD; 
		myDrawMatrix[i].wavex1b = myDrawMatrix[i].windx1b = (int)((sin(w) * myDrawPara.ARROW_LEN_MID)+.5);
		myDrawMatrix[i].wavey1b = myDrawMatrix[i].windy1b = (int)((cos(w) * myDrawPara.ARROW_LEN_MID)+.5);
		myDrawMatrix[i].wavex2 = myDrawMatrix[i].windx2 = (int)((sin(w) * myDrawPara.ARROW_LEN)+.5);
		myDrawMatrix[i].wavey2 = myDrawMatrix[i].windy2 = (int)((cos(w) * myDrawPara.ARROW_LEN)+.5);

		w  = (double)(dir-200) * RAD; 
		myDrawMatrix[i].wavex3 = myDrawMatrix[i].windx3 = (int)((sin(w) * myDrawPara.ARROW_WING)+.5); 
		myDrawMatrix[i].wavey3 = myDrawMatrix[i].windy3 = (int)((cos(w) * myDrawPara.ARROW_WING)+.5);
		
		w=(double)(dir-160.) * RAD; 
		myDrawMatrix[i].wavex4 = myDrawMatrix[i].windx4 = (int)((sin(w) * myDrawPara.ARROW_WING)+.5);
		myDrawMatrix[i].wavey4 = myDrawMatrix[i].windy4 = (int)((cos(w) * myDrawPara.ARROW_WING)+.5);

		myDrawMatrix[i].knots=(int)((double)i*.48595);  //0.25 * 1.9438 = .48595
		w=(double)i*.25;

		myDrawMatrix[i].beaufort=0; 
		if(w != 0.)
			{
			if(w >= 57.0) myDrawMatrix[i].beaufort = 17;
			else if(w >= 51.0) myDrawMatrix[i].beaufort = 16;
			else if(w >= 46.2) myDrawMatrix[i].beaufort = 15; 
			else if(w >= 41.5) myDrawMatrix[i].beaufort = 14;
			else if(w >= 37.0) myDrawMatrix[i].beaufort = 13; 
			else if(w >= 32.7) myDrawMatrix[i].beaufort = 12;
			else if(w >= 28.5) myDrawMatrix[i].beaufort = 11; 
			else if(w >= 24.5) myDrawMatrix[i].beaufort = 10;
			else if(w >= 20.8) myDrawMatrix[i].beaufort = 9;  
			else if(w >= 17.2) myDrawMatrix[i].beaufort = 8;
			else if(w >= 13.9) myDrawMatrix[i].beaufort = 7;  
			else if(w >= 10.8) myDrawMatrix[i].beaufort = 6;
			else if(w >= 8.0) myDrawMatrix[i].beaufort = 5;  
			else if(w >= 5.5) myDrawMatrix[i].beaufort = 4;
			else if(w >= 3.4) myDrawMatrix[i].beaufort = 3;   
			else if(w >= 1.6) myDrawMatrix[i].beaufort = 2;
			else if(w >= 0.3) myDrawMatrix[i].beaufort = 1;
			}
		}	
	planecol[0]=planecol[1]=planecol[2]=planecol[5]=linecol[4]=PAL_COL_SEA;
	planecol[3]=PAL_COL_TIDE;
	planecol[4]=linecol[14]=PAL_COL_LAND;
	planecol[6]=linecol[15]=PAL_COL_BORDER;
	planecol[7]=PAL_COL_RIVER;
	planecol[9]=PAL_COL_COUNTRY;
}


//my little symobols 
void DrawingTools::PlotOjects(CDC* pDC, int obj)
{

	OBJFile.Seek(sizeof(struct BOYHeader)+(obj*sizeof(struct BOYObjectPointer)),CFile::begin);
	OBJFile.Read(&OBJ_PTR,sizeof(struct BOYObjectPointer));

	for(int i=0; i<OBJ_PTR.ObjectCount; i++)
		{
		OBJFile.Seek(OBJ_PTR.ObjectSeek+(i*sizeof(struct BOYObject)),CFile::begin);
		OBJFile.Read(&OBJ_OBJ,sizeof(struct BOYObject));
		PlottObjektLines(pDC);
		}
 
}

void DrawingTools::PlottObjektLines(CDC* pDC)
{
	CBrush  Brush;
	CPen    Pen;
	CPen*	pOldPen;
	CBrush*  pOldBrush;
	int		i;


	COLORREF	BorderColor,FillColor;
	pDC->SetBkMode(TRANSPARENT);
	m_logpen.lopnStyle = PS_INSIDEFRAME;
	m_logpen.lopnWidth.x = m_logpen.lopnWidth.y = 1; 
	m_logbrush.lbStyle = BS_SOLID;
	m_logbrush.lbHatch = HS_HORIZONTAL;
	

	BorderColor=linecol[OBJ_OBJ.Color>>4];
	FillColor=linecol[OBJ_OBJ.Color&15];

	if(OBJ_OBJ.Type & BORDER || OBJ_OBJ.Type&POLYLINE)
			{
			if(OBJ_OBJ.Type&POLYLINE)m_logpen.lopnColor=FillColor;
			else m_logpen.lopnColor=BorderColor;
			if (!Pen.CreatePenIndirect(&m_logpen)) return;
			pOldPen=pDC->SelectObject(&Pen);
			}
	else pOldPen=(CPen*)pDC->SelectStockObject(NULL_PEN);

	if(OBJ_OBJ.Type & FILLED)
			{
			m_logbrush.lbColor=FillColor;
			if (!Brush.CreateBrushIndirect(&m_logbrush)) return;
			pOldBrush=pDC->SelectObject(&Brush);
			}
	else pOldBrush=(CBrush*)pDC->SelectStockObject(NULL_BRUSH);

	OBJFile.Seek(OBJ_OBJ.PointSeek,CFile::begin);
	OBJFile.Read(SPointIbuf,sizeof(struct LinePoint)*OBJ_OBJ.PointCount);
	
	for (i=0; i<OBJ_OBJ.PointCount; i++)
		{
		PointOBuf[i].x=ObjectOffsetX+(int)((double)((double)SPointIbuf[i].x * ObjectScale));
		PointOBuf[i].y=ObjectOffsetY+(int)((double)((double)SPointIbuf[i].y * ObjectScale));
		}
	
	switch (OBJ_OBJ.Type & 15)
		{
		case	POLYLINE:	pDC->Polyline(&PointOBuf[0],OBJ_OBJ.PointCount);
							break;
		case	 POLYGON:	pDC->Polygon(&PointOBuf[0],OBJ_OBJ.PointCount);
							break;
		case	 ELLIPSE:	pDC->Ellipse(PointOBuf[0].x,PointOBuf[0].y,PointOBuf[1].x,PointOBuf[1].y);
							break;
		}

	pDC->SelectObject(pOldPen);
	pDC->SelectObject(pOldBrush);
}

void DrawingTools::SetGridSize(int size, int force)
{
	myBMS_Para.GridSize=size;
	myBMS_Para.refresch=1;

	DRAW_PARA &DrawPara=GetDrawPara();

	switch(size)
		{
		default:	DrawPara.GRIDWIDTH=30;		
					DrawPara.GRIDMIDDLE=15;	
					DrawPara.SYMBOLGRID=30;	
					DrawPara.SYMBOLMIDDEL=0;
					DrawPara.SYMBOLHALFMID=15;
					DrawPara.SYMBOLTOP=10;
					DrawPara.ARROW_LEN=12.5;		
					DrawPara.ARROW_LEN_MID=3.5;	
					DrawPara.ARROW_WING=5.0;	
					DrawPara.FIG_HEIGHT=-8;
					DrawPara.FIG_WEIGHT=FW_NORMAL;
					DrawPara.ObjectScale=.02;
					break;
		case 1:		DrawPara.GRIDWIDTH=41;		
					DrawPara.GRIDMIDDLE=20;	
					DrawPara.SYMBOLGRID=41;	
					DrawPara.SYMBOLMIDDEL=0;
					DrawPara.SYMBOLHALFMID=20;
					DrawPara.SYMBOLTOP=13;
					DrawPara.ARROW_LEN=16.6666;		
					DrawPara.ARROW_LEN_MID=5.0;	
					DrawPara.ARROW_WING=7.5;
					DrawPara.FIG_HEIGHT=-10;
					DrawPara.FIG_WEIGHT=FW_NORMAL;
					DrawPara.ObjectScale=.025;
					break;
		case 2:		DrawPara.GRIDWIDTH=51;		
					DrawPara.GRIDMIDDLE=25;	
					DrawPara.SYMBOLGRID=51;	
					DrawPara.SYMBOLMIDDEL=0;
					DrawPara.SYMBOLHALFMID=25;
					DrawPara.SYMBOLTOP=17;
					DrawPara.ARROW_LEN=20.833;		
					DrawPara.ARROW_LEN_MID=6.5;	
					DrawPara.ARROW_WING=9.0;	
					DrawPara.FIG_HEIGHT=-12;
					DrawPara.FIG_WEIGHT=FW_NORMAL;
					DrawPara.ObjectScale=.03;
					break;
		case 3:		//myDrawPara must be set before
					
					break;

		}

	if(force) InitTheWorkMatrixes();
}
