// DrawingTools.h 
//
/////////////////////////////////////////////////////////////////////////////
#if !defined(DRAWINGTOOLS)
#define DRAWINGTOOLS

#include "BMSD_Defs.h"


#define UBYTE unsigned char


typedef struct
		{
			//dot positions of the wind direction arrow
		int			windx1,windy1;		//move to point 1
		int			windx1a,windy1a;	//move to point 1a
		int			windx1b,windy1b;	//move to point 1b
		int			windx2,windy2;		//line to point 2
		int			windx3,windy3;		//arrow left wing
		int			windx4,windy4;		//arrow right wing

			//dot positions of the wave direction arrow
		int			wavex1,wavey1;		//move to point 1
		int			wavex1a,wavey1a;		//move to point 1
		int			wavex1b,wavey1b;		//move to point 1
		int			wavex2,wavey2;		//wave top point 2
		int			wavex3,wavey3;		//left wave wing
		int			wavex4,wavey4;		//right wave wing

			// windspeed transform value = index * .025 m/s
		int			beaufort;			//beaufort value
		int			knots;				//windspeed knots value 

		
			//colors index
		COLOR_MATRIX col;

		}DRAW_MATRIX;

typedef	CArray<DRAW_MATRIX,DRAW_MATRIX&> DrawingMatrix;
typedef	CArray<POINT,POINT&> PolyPoints;

// I know, this is german
#define OBJ_LOGO	0
#define OBJ_CLOUDS	1
#define OBJ_SUN		2
#define OBJ_RAIN	3
#define OBJ_HAIL	4
#define OBJ_THUNDER 5
#define OBJ_SNOW	6
#define OBJ_CLOUDSUN 7


#define RGB_SCHWARZ 	RGB(0,0,0)	
#define RGB_WEISS		RGB(255,255,255)
#define RGB_BLAUGRAU	RGB(0,0,208)
#define RGB_CYANBLAU	RGB(0,80,224)

#define POLYLINE		0x02
#define POLYGON			0x04
#define ELLIPSE			0x08

#define FILLED			0x10
#define	BORDER			0x20

#define COL_P_FRAME		myDrawMatrix[9].col.bathy


class DrawingTools
{
public: // create from serialization only
	DrawingTools();
	virtual ~DrawingTools();

public:

	COLORREF		PTCol;			//Druck Textcolor
	COLORREF		PFCol;			//Druck Framecolor
	COLORREF		TTCol;			//Temperatur Textcolor
	COLORREF		TFCol;			//Temperatur Framecolor
	COLORREF		WTCol;			//Wind Textcolor
	COLORREF		WFCol;			//Wind Framecolor
	COLORREF		WaTCol;			//Wave Textcolor
	COLORREF		WaFCol;			//Wave Framecolor
	int				DetailFlags,SynopDetailFlags;
	DrawingMatrix	myDrawMatrix;
	

	CFile					OBJFile;
	CFileException			fex;
	struct BOYObject		OBJ_OBJ;
	struct BOYHeader		OBJ_HDR;
	struct BOYObjectPointer OBJ_PTR;
	double					ObjectScale;
	int						ObjectOffsetX,ObjectOffsetY;
	struct	LinePoint		*SPointIbuf;
	CPoint					PointOBuf[8096];
	struct POINT_NEWLINE	PointIBuf[8096];


	CPen		*WFPen,*WTPen;
	LOGPEN		m_logpen;
	LOGBRUSH	m_logbrush;

	short		Screen_ymin;
	short		Screen_xmin;
	short		Screen_ymax;
	short		Screen_xmax;

	CRect		ViewRect;
	double		xmapscale, ymapscale;		//Nav_Map Skalierung
	double		lmapxoff;		//Nav_Map X-Offset
	double		lmapyoff;		//Nav_Map Y-Offset
	int			swh,shh;

	CFileException	le,we;
	CFile		lh,wh;
	CSize		sizeTotal;
	int			ClipMax;

	int			N_hour;
	time_t		N_date;
	double		N_xscale,N_yscale;
	double		N_xoff;
	double		N_yoff;

	int			grid_xx,grid_yy,initColors;


	ScreenGridData	sc_data;

	int			GetRect(LPRECT rect);

	void		DrawSeaData(CDC* pDC,LPRECT rect, time_t reftime, int hour, int details,
								float xscale, float yscale, float xoff, float yoff);
	void		DrawSfcData(CDC* pDC,LPRECT rect, time_t reftime, int hour, int details,
								float xscale, float yscale, float xoff, float yoff);

	void		CalcPoint(COORDIANTE *in_coor, POINT *out_point);

	void		SetPreciAndClouds(CDC* pDC,int z);
	void		Arrow(CDC* pDC,int z);
	void		Wave(CDC* pDC,int z);
	void		DrawPlanes(CDC* pDC, int type);
	void		DrawBathymetry(CDC* pDC,int set=NULL);
	void		DrawPlaneLine(CDC* pDC, int type, COLORREF col);
	void		DrawPlaneColLine(CDC* pDC, int type);
	void		DrawIsoLine(CDC* pDC, COORDIANTE *point, int count, COLORREF col);
	void		DrawIsoPolygon(CDC* pDC, COORDIANTE *point, int count, COLORREF bcol, COLORREF pcol=NULL, int flag=0);
	void		InitTheWorkMatrixes();
	void		PlottObjektLines(CDC* pDC);
	void		PlotOjects(CDC* pDC, int obj);

	void		DoppelText(CDC* pDC, char *txt, int x, int y, COLORREF TCol, COLORREF FCol);
	void		WriteText(CDC* pDC,char *txt, int x, int y, COLORREF TCol);


	void		DrawSynData(CDC* pDC, LPRECT rect, time_t reftime, int hour, int details,
								float xscale, float yscale, float xoff, float yoff);

	void		SetDetailSympol(CDC* pDC, int rflags, int prmsl, int btcdc, int temp,int z,int ugrd=0, int vgrd=0);
	void		Precipitation(CDC* pDC, int rflags, int btcdc);
	void		SynGridArrow(CDC* pDC, int ugrd, int vgrd);
	void		SynGridArrow(CDC* pDC, int z);
	void		SetGridSize(int size=0, int force=1);


};

#endif //DRAWINGTOOLS